package p08_funkcje;

import java.math.BigInteger;
import java.util.Scanner;
import java.util.stream.LongStream;

public class Silnia {
    
    // Funkcja zwraca silnię z podanej liczby.
    // silnia to jest iloczyn liczb naturalnych od 1 do n włącznie
    // np. silnia(5) = 1*2*3*4*5 = 120
    // silnia(0) = 1
    static long silnia(int n) {
        long iloczyn = 1;
        for(int i=2; i <= n; i++) {
            iloczyn = iloczyn * i;  // albo iloczyn *= i;
        }
        return iloczyn;
    }
    
    static BigInteger silniaBig(int n) {
        BigInteger iloczyn = BigInteger.ONE;
        for(int i=2; i <= n; i++) {
            iloczyn = iloczyn.multiply(BigInteger.valueOf(i));
        }
        return iloczyn;
    }
    
    // wersja rekurencyjna
    // rekurencja (recursion) to technika, w której funkcja wywołuje samą siebie
    // w praktyce rekurencji używa się rzadko, ale jest ona ważna w teorii programowania
    static long silniaR(int n) {
        if(n <= 1) return 1;
        return silniaR(n-1) * n;
    }
    
    // Od Java 8 można też zastosować "techniki funkcyjne"
    // a dokładnie schemat z redukcją elementów strumienia
    static long silniaFun(int n) {
        return LongStream.rangeClosed(1, n).reduce(1, (a,b) -> a*b);
    }
    
    static BigInteger silniaBigFun(int n) {
        return LongStream.rangeClosed(1, n)
                .mapToObj(BigInteger::valueOf)
                .reduce(BigInteger.ONE, BigInteger::multiply);
    }

    public static void main(String[] args) {
        Scanner scanner = new Scanner(System.in);
        System.out.println("Podawaj liczby, a ja będę liczyć silnię.");
        System.out.println("Aby zakończyć, wpisz liczbę ujemną.");
        while(true) {
            System.out.print("Podaj argument: ");
            int arg = scanner.nextInt();
            if(arg < 0) break;
            var wynik = silniaBigFun(arg);
            System.out.printf("silnia(%s) = %s%n", arg, wynik);
        }
    }

}
